
/*
 * Copyright (c) Developers of Easy Diff
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>				<changed by>		<summary of changes>
 * 07 September 2004	Jonee Ryan Ty		First documented history
 * 14 October 2004		Jonee Ryan Ty		Tried to fix critical bug in the presentation. Strings are
 * 									escaped to be HTML capable.
 * 04 December 2004	Jonee Ryan Ty		Added some styles and improved the presentation
 */

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

public class Diff {
	private static Vector max = new Vector(), vf1, vf2;

	private static int size1, size2;
	private static final boolean MY_VERSION = false;	// if this is true, it would overwrite files without warning

	public static void main(String args[]) {
		System.out.println("\nEasy Diff (c) GPL");

		if (args.length < 2) {
			System.err.println(
				"Program usage: Diff <fileName1> <fileName2> {optional outputFile}\n");
			System.exit(1);
		}

		String outputFile;

		if (args.length > 2)
			outputFile = args[2];
		else
			outputFile = "output.html";

		vf1 = readFile(args[0], true);
		vf2 = readFile(args[1], true);

		File f = new File(outputFile);

		if (f.exists() && !MY_VERSION) {
			System.err.println(
				"\nFile exists: "
					+ outputFile
					+ "\nDelete it first or specify a different file name.\n");
			System.exit(1);
		}

		diff();
		present(args[0], args[1], f); // would present the solution in html
	}

	private static void diff() {
		Vector hits = new Vector();

		/*1*/
		System.out.println("\nFirst part follows:");
		System.out.println("i H[i]");

		size1 = vf1.size();
		size2 = vf2.size();

		int tmpHit[], count = 0;

		String curLine1, curLine2;

		for (int a = 0; a < size1; a++) {
			curLine1 = (String) vf1.elementAt(a);

			for (int b = 0; b < size2; b++) {
				curLine2 = (String) vf2.elementAt(b);

				if (curLine1.equals(curLine2)) {
					System.out.println(count++ +" (" + a + ", " + b + ")");
					tmpHit = new int[2];
					tmpHit[0] = a;
					tmpHit[1] = b;
					hits.add(tmpHit);
				}
			}
		}

		/*2*/
		System.out.println("\nSecond part follows:");
		System.out.println("i A[i]");

		if (hits.size() == 0)
			return;

		int largestSub[] = new int[hits.size()];
		for (int c = 0; c < largestSub.length; c++)
			largestSub[c] = 1;

		int cur[], next[];

		for (int d = largestSub.length - 2; d > -1; d--) {
			cur = (int[]) hits.elementAt(d);

			for (int e = d + 1; e < largestSub.length; e++)
				if (largestSub[e] >= largestSub[d]) {
					next = (int[]) hits.elementAt(e);

					if ((cur[0] != next[0]) && (cur[1] < next[1]))
						largestSub[d] = largestSub[e] + 1;
				}
		}

		for (int a = largestSub.length - 1; a > 0; a--)
			System.out.println(a + " " + largestSub[a]);

		/*3*/
		System.out.println("\nThird part follows:");
		System.out.println("i A[i] \tH[i]");

		int maxLength = largestSub[0], maxIndx = 0, curHit[];

		for (int g = 1; g < largestSub.length; g++)
			if (largestSub[g] > maxLength) {
				maxLength = largestSub[g];
				maxIndx = g;
			}
		// the last hit, variable re-used
		tmpHit = (int[]) hits.elementAt(maxIndx);

		System.out.println(
			maxIndx
				+ " "
				+ maxLength
				+ " \t("
				+ tmpHit[0]
				+ ", "
				+ tmpHit[1]
				+ ") ");

		max.add(tmpHit);
		maxLength--;

		for (int f = maxIndx + 1; maxLength > 0; f++)
			if (largestSub[f] == maxLength) {
				curHit = (int[]) hits.elementAt(f);

				if ((tmpHit[0] == curHit[0]) || (tmpHit[1] >= curHit[1]))
					continue;

				else {
					tmpHit = (int[]) hits.elementAt(f);

					System.out.println(
						f
							+ " "
							+ maxLength
							+ " \t("
							+ tmpHit[0]
							+ ", "
							+ tmpHit[1]
							+ ") ");

					max.add(tmpHit);
					maxLength--;
				}
			}

	} // diff

	private static void present(
		String diffFile1,
		String diffFile2,
		File outputFile) {

		FileOutputStream fileOutputStream = null;
		PrintStream output = null;		
		int prevHit[] = null, nextHit[] = null, curRow1 = 0, curRow2 = 0, streak = 1;
		boolean exceedOnce = true;

		try {
			fileOutputStream = new FileOutputStream(outputFile);
			output = new PrintStream(fileOutputStream);

			output.print("<html>");
			output.print("\n\t<head>");
			output.print(
				"\n\t\t<title>Diff: "
					+ diffFile1
					+ ", "
					+ diffFile2
					+ "</title>");

			output.print("\n\t\t<style>");
			output.print("\n\t\t\ttable.sideBox {");
			output.print("\n\t\t\t\tbackground-color:#dddddd");
			output.print("\n\t\t\t\tfont-family:arial;");
			output.print("\n\t\t\t}");
			output.print("\n\t\t</style>");

			output.print("\n\t</head>");

			output.print("\n\t<body>");
			output.print(
				"\n\t\t<table width=100% border=0 cellpadding=2 cellspacing=2 class=sideBox>");
			output.print("\n\t\t\t<tr bgcolor=#E0FFFF>");
			output.print("\n\t\t\t\t<td align=center>" + diffFile1 + "</td>");
			output.print("\n\t\t\t\t<td align=center>" + diffFile2 + "</td>");
			output.print("\n\t\t\t</tr>");

			Enumeration enum = max.elements();

			if (enum.hasMoreElements()) {
				prevHit = (int[]) enum.nextElement();

				do {
					if (enum.hasMoreElements())
						nextHit = (int[]) enum.nextElement();
					else {
						nextHit = null;
						exceedOnce = false;
					}
					
					if (nextHit != null) {
						if ((prevHit[0] + streak == nextHit[0]) && (prevHit[1] + streak == nextHit[1])) {
							streak = streak + 1;
							continue;
						} 
					}
											
					if ((prevHit[0] != curRow1) || (prevHit[1] != curRow2)) {
						output.print("\n\t\t\t<tr bgcolor=#FFCC66 valign=top>");

						if (prevHit[0] != curRow1) {
							output.print("\n\t\t\t\t<td width=50%>\n");
							for (; curRow1 < prevHit[0]; curRow1++)
								output.print(
									StringUtil.cleanHTMLStr(
										(String) vf1.elementAt(curRow1))
										+ "<br>\n");
							output.print("\n\t\t\t\t</td>");

						} else
							output.print("\n\t\t\t\t<td width=50%>&nbsp;</td>");

						if (prevHit[1] != curRow2) {
							output.print("\n\t\t\t\t<td width=50%>\n");
							for (; curRow2 < prevHit[1]; curRow2++)
								output.print(
									StringUtil.cleanHTMLStr(
										(String) vf2.elementAt(curRow2))
										+ "<br>\n");
							output.print("\n\t\t\t\t</td>");

						} else
							output.print("\n\t\t\t\t<td width=50%>&nbsp;</td>");

						output.print("\n\t\t\t</tr>");
					}
										
					output.print("\n\t\t\t<tr bgcolor=#FFFFCC valign=top>");

					output.print("\n\t\t\t\t<td width=50%>");
					for (int i = 0; i < streak; i++)
						output.print(StringUtil.cleanHTMLStr((String)vf1.elementAt(prevHit[0] + i)) + "<br>");
					output.print("\n\t\t\t\t</td>");

					output.print("\n\t\t\t\t<td width=50%>");
					for (int j = 0; j < streak; j++)
						output.print(StringUtil.cleanHTMLStr((String)vf2.elementAt(prevHit[1] + j)) + "<br>");
					output.print("\n\t\t\t\t</td>");

					output.print("\n\t\t\t</tr>");
					
					curRow1 = prevHit[0] + streak;
					curRow2 = prevHit[1] + streak;
					
					streak = 1;
					
					prevHit = nextHit;

				} while (enum.hasMoreElements() || exceedOnce);
			}

			if ((curRow1 < size1) || (curRow2 < size2)) {
				output.print("\n\t\t\t<tr bgcolor=#FFCC66 valign=top>");

				output.print("\n\t\t\t\t<td width=50%>\n");
				for (; curRow1 < size1; curRow1++)
					output.print(
						StringUtil.cleanHTMLStr(
							(String) vf1.elementAt(curRow1))
							+ "<br>\n");
				output.print("\n\t\t\t\t</td>");

				output.print("\n\t\t\t\t<td width=50%>\n");
				for (; curRow2 < size2; curRow2++)
					output.print(
						StringUtil.cleanHTMLStr(
							(String) vf2.elementAt(curRow2))
							+ "<br>\n");
				output.print("\n\t\t\t\t</td>");

				output.print("\n\t\t\t</tr>");
			}

			output.print("\n\t\t</table>");
			output.print("\n\t</body>");
			output.print("\n</html>");
			output.flush();
			fileOutputStream.close();
			output.close();

			System.out.println(
				"\nYou may now read the contents of " + outputFile + "\n");

		} catch (IOException ioEx) {
			System.err.println("exception: " + ioEx.getMessage() + "\n");
			System.exit(1);
		}
	}

	private static Vector readFile(String fileName, boolean isTrim) {
		File f = new File(fileName);
		FileReader fileReader;
		BufferedReader buf;
		Vector ret = new Vector();
		String tmpStr;

		if (!f.exists()) {
			System.err.println("file not found: " + fileName + "\n");
			System.exit(1);
		}

		try {
			fileReader = new FileReader(f);
			buf = new BufferedReader(fileReader);

			while ((tmpStr = buf.readLine()) != null) {
				if (isTrim)
					ret.add(tmpStr.trim());
				else
					ret.add(tmpStr);
			}

			fileReader.close();
			buf.close();
		} catch (IOException ioEx) {
			System.err.println("exception: " + ioEx.getMessage() + "\n");
			System.exit(1);
		}

		return ret;
	} // readFile

}
